@@ -4,9 +4,6 @@ from django.utils import translation
 from django.db.models import ObjectDoesNotExist
 from pybb import util
 
-from pybb.signals import user_saved
-
-
 class PybbMiddleware(object):
     def process_request(self, request):
         if request.user.is_authenticated():
@@ -18,6 +15,9 @@ class PybbMiddleware(object):
             except ObjectDoesNotExist:
                 # Ok, we should create new profile for this user
                 # and grant permissions for add posts
+                # It should be caused rarely, so we move import signal here
+                # to prevent circular import
+                from pybb.models import user_saved
                 user_saved(request.user, created=True)
                 profile = util.get_pybb_profile(request.user)
 
