@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
+import functools
 from django.contrib.auth.models import Permission
 from django.db.models.signals import post_delete, post_save
+from pybb.profiles import PybbProfile
 from pybb.subscription import notify_topic_subscribers
 
 import os.path
@@ -11,15 +13,11 @@ from django.core.urlresolvers import reverse
 from django.db.utils import IntegrityError
 from django.utils.html import strip_tags
 from django.utils.translation import ugettext_lazy as _
-from django.conf import settings
 from django.utils.timezone import now as tznow
 
 from annoying.fields import AutoOneToOneField
-try:
-    from sorl.thumbnail import ImageField
-except ImportError:
-    from django.db.models import ImageField
-from pybb.util import unescape, get_user_model, get_username_field, get_pybb_profile_model, get_pybb_profile
+
+from pybb.util import unescape, get_user_model, get_username_field, get_pybb_profile_model, get_pybb_profile, get_file_path
 
 User = get_user_model()
 username_field = get_username_field()
@@ -44,31 +42,6 @@ except ImportError:
     from django.db.transaction import commit_on_success as atomic_func
 
 
-TZ_CHOICES = [(float(x[0]), x[1]) for x in (
-(-12, '-12'), (-11, '-11'), (-10, '-10'), (-9.5, '-09.5'), (-9, '-09'),
-(-8.5, '-08.5'), (-8, '-08 PST'), (-7, '-07 MST'), (-6, '-06 CST'),
-(-5, '-05 EST'), (-4, '-04 AST'), (-3.5, '-03.5'), (-3, '-03 ADT'),
-(-2, '-02'), (-1, '-01'), (0, '00 GMT'), (1, '+01 CET'), (2, '+02'),
-(3, '+03'), (3.5, '+03.5'), (4, '+04'), (4.5, '+04.5'), (5, '+05'),
-(5.5, '+05.5'), (6, '+06'), (6.5, '+06.5'), (7, '+07'), (8, '+08'),
-(9, '+09'), (9.5, '+09.5'), (10, '+10'), (10.5, '+10.5'), (11, '+11'),
-(11.5, '+11.5'), (12, '+12'), (13, '+13'), (14, '+14'),
-)]
-
-
-#noinspection PyUnusedLocal
-def get_file_path(instance, filename, to='pybb/avatar'):
-    """
-    This function generate filename with uuid4
-    it's useful if:
-    - you don't want to allow others to see original uploaded filenames
-    - users can upload images with unicode in filenames wich can confuse browsers and filesystem
-    """
-    ext = filename.split('.')[-1]
-    filename = "%s.%s" % (uuid.uuid4(), ext)
-    return os.path.join(to, filename)
-
-
 class Category(models.Model):
     name = models.CharField(_('Name'), max_length=80)
     position = models.IntegerField(_('Position'), blank=True, default=0)
@@ -101,6 +74,8 @@ class Category(models.Model):
 
 class Forum(models.Model):
     category = models.ForeignKey(Category, related_name='forums', verbose_name=_('Category'))
+    parent = models.ForeignKey('self', related_name='child_forums', verbose_name=_('Parent forum'),
+                               blank=True, null=True)
     name = models.CharField(_('Name'), max_length=80)
     position = models.IntegerField(_('Position'), blank=True, default=0)
     description = models.TextField(_('Description'), blank=True)
@@ -150,7 +125,12 @@ class Forum(models.Model):
         """
         Used in templates for breadcrumb building
         """
-        return self.category,
+        parents = [self.category]
+        parent = self.parent
+        while parent is not None:
+            parents.insert(1, parent)
+            parent = parent.parent
+        return parents
 
 
 class Topic(models.Model):
@@ -239,7 +219,9 @@ class Topic(models.Model):
         """
         Used in templates for breadcrumb building
         """
-        return self.forum.category, self.forum
+        parents = self.forum.get_parents()
+        parents.append(self.forum)
+        return parents
 
     def poll_votes(self):
         if self.poll_type != self.POLL_TYPE_NONE:
@@ -337,47 +319,6 @@ class Post(RenderableItem):
         return self.topic.forum.category, self.topic.forum, self.topic,
 
 
-class PybbProfile(models.Model):
-    """
-    Abstract class for user profile, site profile should be inherted from this class
-    """
-
-    class Meta(object):
-        abstract = True
-        permissions = (
-            ("block_users", "Can block any user"),
-        )
-
-    signature = models.TextField(_('Signature'), blank=True,
-        max_length=defaults.PYBB_SIGNATURE_MAX_LENGTH)
-    signature_html = models.TextField(_('Signature HTML Version'), blank=True,
-        max_length=defaults.PYBB_SIGNATURE_MAX_LENGTH + 30)
-    time_zone = models.FloatField(_('Time zone'), choices=TZ_CHOICES,
-        default=float(defaults.PYBB_DEFAULT_TIME_ZONE))
-    language = models.CharField(_('Language'), max_length=10, blank=True,
-        choices=settings.LANGUAGES,
-        default=settings.LANGUAGE_CODE)
-    show_signatures = models.BooleanField(_('Show signatures'), blank=True,
-        default=True)
-    post_count = models.IntegerField(_('Post count'), blank=True, default=0)
-    avatar = ImageField(_('Avatar'), blank=True, null=True,
-        upload_to=get_file_path)
-    autosubscribe = models.BooleanField(_('Automatically subscribe'),
-        help_text=_('Automatically subscribe to topics that you answer'),
-        default=defaults.PYBB_DEFAULT_AUTOSUBSCRIBE)
-
-    def save(self, *args, **kwargs):
-        self.signature_html = defaults.PYBB_MARKUP_ENGINES[defaults.PYBB_MARKUP](self.signature)
-        super(PybbProfile, self).save(*args, **kwargs)
-
-    @property
-    def avatar_url(self):
-        try:
-            return self.avatar.url
-        except:
-            return defaults.PYBB_DEFAULT_AVATAR_URL
-
-
 class Profile(PybbProfile):
     """
     Profile class that can be used if you doesn't have
@@ -402,7 +343,7 @@ class Attachment(models.Model):
     post = models.ForeignKey(Post, verbose_name=_('Post'), related_name='attachments')
     size = models.IntegerField(_('Size'))
     file = models.FileField(_('File'),
-                            upload_to=lambda instance, filename: get_file_path(instance, filename, to=defaults.PYBB_ATTACHMENT_UPLOAD_TO))
+                            upload_to=functools.partial(get_file_path, to=defaults.PYBB_ATTACHMENT_UPLOAD_TO))
 
     def save(self, *args, **kwargs):
         self.size = self.file.size
