@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
-import time, datetime
+import time
+import datetime
 import os
 
 from django.contrib.auth.models import Permission
@@ -68,6 +69,7 @@ class FeaturesTest(TestCase, SharedTestModule):
 
     def test_base(self):
         # Check index page
+        Forum.objects.create(name='xfoo1', description='bar1', category=self.category, parent=self.forum)
         url = reverse('pybb:index')
         response = self.client.get(url)
         parser = html.HTMLParser(encoding='utf8')
@@ -76,6 +78,7 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertContains(response, self.forum.get_absolute_url())
         self.assertTrue(defaults.PYBB_DEFAULT_TITLE in tree.xpath('//title')[0].text_content())
         self.assertEqual(len(response.context['categories']), 1)
+        self.assertEqual(len(response.context['categories'][0].forums_accessed), 1)
 
     def test_forum_page(self):
         # Check forum page
@@ -88,9 +91,11 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertFalse(response.context['is_paginated'])
 
     def test_category_page(self):
+        Forum.objects.create(name='xfoo1', description='bar1', category=self.category, parent=self.forum)
         response = self.client.get(self.category.get_absolute_url())
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, self.forum.get_absolute_url())
+        self.assertEqual(len(response.context['object'].forums_accessed), 1)
 
     def test_profile_language_default(self):
         user = User.objects.create_user(username='user2', password='user2', email='user2@example.com')
@@ -354,6 +359,20 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertEqual(ForumReadTracker.objects.filter(user=self.user).count(), 1)
         self.assertEqual(ForumReadTracker.objects.filter(user=self.user, forum=self.forum).count(), 1)
 
+    def test_read_tracker_after_posting(self):
+        client = Client()
+        client.login(username='zeus', password='zeus')
+        add_post_url = reverse('pybb:add_post', kwargs={'topic_id': self.topic.id})
+        response = client.get(add_post_url)
+        values = self.get_form_values(response)
+        values['body'] = 'test tracking'
+        response = client.post(add_post_url, values, follow=True)
+
+        # after posting in topic it should be readed
+        # because there is only one topic, so whole forum should be marked as readed
+        self.assertEqual(TopicReadTracker.objects.filter(user=self.user, topic=self.topic).count(), 0)
+        self.assertEqual(ForumReadTracker.objects.filter(user=self.user, forum=self.forum).count(), 1)
+
     def test_pybb_is_topic_unread_filter(self):
         forum_1 = self.forum
         topic_1 = self.topic
@@ -411,6 +430,54 @@ class FeaturesTest(TestCase, SharedTestModule):
             [t.unread for t in pybb_topic_unread([topic_1, topic_2, topic_3], user_ann)],
             [False, False, False])
 
+    def test_is_forum_unread_filter(self):
+        Forum.objects.all().delete()
+
+        forum_parent = Forum.objects.create(name='f1', category=self.category)
+        forum_child1 = Forum.objects.create(name='f2', category=self.category, parent=forum_parent)
+        forum_child2 = Forum.objects.create(name='f3', category=self.category, parent=forum_parent)
+        topic_1 = Topic.objects.create(name='topic_1', forum=forum_parent, user=self.user)
+        topic_2 = Topic.objects.create(name='topic_2', forum=forum_child1, user=self.user)
+        topic_3 = Topic.objects.create(name='topic_3', forum=forum_child2, user=self.user)
+
+        Post(topic=topic_1, user=self.user, body='one').save()
+        Post(topic=topic_2, user=self.user, body='two').save()
+        Post(topic=topic_3, user=self.user, body='three').save()
+
+        user_ann = User.objects.create_user('ann', 'ann@localhost', 'ann')
+        client_ann = Client()
+        client_ann.login(username='ann', password='ann')
+
+        forum_parent = Forum.objects.get(id=forum_parent.id)
+        forum_child1 = Forum.objects.get(id=forum_child1.id)
+        forum_child2 = Forum.objects.get(id=forum_child2.id)
+        self.assertListEqual([f.unread for f in pybb_forum_unread([forum_parent, forum_child1, forum_child2], user_ann)],
+                             [True, True, True])
+
+        # unless we read parent topic, there is unreaded topics in child forums
+        client_ann.get(topic_1.get_absolute_url())
+        forum_parent = Forum.objects.get(id=forum_parent.id)
+        forum_child1 = Forum.objects.get(id=forum_child1.id)
+        forum_child2 = Forum.objects.get(id=forum_child2.id)
+        self.assertListEqual([f.unread for f in pybb_forum_unread([forum_parent, forum_child1, forum_child2], user_ann)],
+                             [True, True, True])
+
+        # still unreaded topic in one of the child forums
+        client_ann.get(topic_2.get_absolute_url())
+        forum_parent = Forum.objects.get(id=forum_parent.id)
+        forum_child1 = Forum.objects.get(id=forum_child1.id)
+        forum_child2 = Forum.objects.get(id=forum_child2.id)
+        self.assertListEqual([f.unread for f in pybb_forum_unread([forum_parent, forum_child1, forum_child2], user_ann)],
+                             [True, False, True])
+
+        # all topics readed
+        client_ann.get(topic_3.get_absolute_url())
+        forum_parent = Forum.objects.get(id=forum_parent.id)
+        forum_child1 = Forum.objects.get(id=forum_child1.id)
+        forum_child2 = Forum.objects.get(id=forum_child2.id)
+        self.assertListEqual([f.unread for f in pybb_forum_unread([forum_parent, forum_child1, forum_child2], user_ann)],
+                             [False, False, False])
+
     def test_read_tracker_when_topics_forum_changed(self):
         forum_1 = Forum.objects.create(name='f1', description='bar', category=self.category)
         forum_2 = Forum.objects.create(name='f2', description='bar', category=self.category)
@@ -684,7 +751,9 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertEqual(Post.objects.filter(user=user).count(), 0)
 
     def test_user_unblocking(self):
-        user = User.objects.create_user('test', 'test@localhost', 'test', is_active=False)
+        user = User.objects.create_user('test', 'test@localhost', 'test')
+        user.is_active=False
+        user.save()
         self.user.is_superuser = True
         self.user.save()
         self.login_client()
@@ -1524,4 +1593,4 @@ class LogonRedirectTest(TestCase, SharedTestModule):
         r = self.get_with_user(self.post.get_absolute_url(), 'staff', 'staff')
         self.assertEquals(r.status_code, 301)
 
-    
\ No newline at end of file
+    
