@@ -4,7 +4,7 @@ import math
 
 from django.contrib.auth.decorators import login_required
 from django.core.cache import cache
-from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
+from django.core.exceptions import PermissionDenied
 from django.core.urlresolvers import reverse
 from django.contrib import messages
 from django.db.models import F, Q
@@ -41,6 +41,7 @@ from pybb import util
 User = util.get_user_model()
 username_field = util.get_username_field()
 
+
 class RedirectToLoginMixin(object):
     """ mixin which redirects to settings.LOGIN_URL if the view encounters an PermissionDenied exception
         and the user is not authenticated. Views inheriting from this need to implement
@@ -70,7 +71,7 @@ class IndexView(generic.ListView):
         ctx = super(IndexView, self).get_context_data(**kwargs)
         categories = ctx['categories']
         for category in categories:
-            category.forums_accessed = perms.filter_forums(self.request.user, category.forums.all())
+            category.forums_accessed = perms.filter_forums(self.request.user, category.forums.filter(parent=None))
         ctx['categories'] = categories
         return ctx
 
@@ -97,7 +98,7 @@ class CategoryView(RedirectToLoginMixin, generic.DetailView):
 
     def get_context_data(self, **kwargs):
         ctx = super(CategoryView, self).get_context_data(**kwargs)
-        ctx['category'].forums_accessed = perms.filter_forums(self.request.user, ctx['category'].forums.all())
+        ctx['category'].forums_accessed = perms.filter_forums(self.request.user, ctx['category'].forums.filter(parent=None))
         ctx['categories'] = [ctx['category']]
         return ctx
 
@@ -115,6 +116,7 @@ class ForumView(RedirectToLoginMixin, generic.ListView):
     def get_context_data(self, **kwargs):
         ctx = super(ForumView, self).get_context_data(**kwargs)
         ctx['forum'] = self.forum
+        ctx['forum'].forums_accessed = perms.filter_forums(self.request.user, self.forum.child_forums.all())
         return ctx
 
     def get_queryset(self):
@@ -205,7 +207,7 @@ class TopicView(RedirectToLoginMixin, generic.ListView):
                                             topic=self.topic)
             else:
                 ctx['form'] = PostForm(topic=self.topic)
-            self.mark_read(self.request, self.topic)
+            self.mark_read(self.request.user, self.topic)
         elif defaults.PYBB_ENABLE_ANONYMOUS_POST:
             ctx['form'] = PostForm(topic=self.topic)
         else:
@@ -225,27 +227,29 @@ class TopicView(RedirectToLoginMixin, generic.ListView):
 
         return ctx
 
-    def mark_read(self, request, topic):
+    def mark_read(self, user, topic):
         try:
-            forum_mark = ForumReadTracker.objects.get(forum=topic.forum, user=request.user)
-        except ObjectDoesNotExist:
+            forum_mark = ForumReadTracker.objects.get(forum=topic.forum, user=user)
+        except ForumReadTracker.DoesNotExist:
             forum_mark = None
         if (forum_mark is None) or (forum_mark.time_stamp < topic.updated):
             # Mark topic as readed
-            topic_mark, new = TopicReadTracker.objects.get_or_create_tracker(topic=topic, user=request.user)
+            topic_mark, new = TopicReadTracker.objects.get_or_create_tracker(topic=topic, user=user)
             if not new:
                 topic_mark.save()
 
             # Check, if there are any unread topics in forum
-            read = Topic.objects.filter(Q(forum=topic.forum) & (Q(topicreadtracker__user=request.user,
-                                                                  topicreadtracker__time_stamp__gt=F('updated'))) |
-                                                                Q(forum__forumreadtracker__user=request.user,
-                                                                  forum__forumreadtracker__time_stamp__gt=F('updated')))
-            unread = Topic.objects.filter(forum=topic.forum).exclude(id__in=read)
-            if not unread.exists():
+            readed = topic.forum.topics.filter((Q(topicreadtracker__user=user,
+                                                  topicreadtracker__time_stamp__gte=F('updated'))) |
+                                                Q(forum__forumreadtracker__user=user,
+                                                  forum__forumreadtracker__time_stamp__gte=F('updated')))\
+                                       .only('id').order_by()
+
+            not_readed = topic.forum.topics.exclude(id__in=readed)
+            if not not_readed.exists():
                 # Clear all topic marks for this forum, mark forum as readed
-                TopicReadTracker.objects.filter(user=request.user, topic__forum=topic.forum).delete()
-                forum_mark, new = ForumReadTracker.objects.get_or_create_tracker(forum=topic.forum, user=request.user)
+                TopicReadTracker.objects.filter(user=user, topic__forum=topic.forum).delete()
+                forum_mark, new = ForumReadTracker.objects.get_or_create_tracker(forum=topic.forum, user=user)
                 forum_mark.save()
 
 
@@ -309,35 +313,6 @@ class AddPostView(PostEditMixin, generic.CreateView):
 
     template_name = 'pybb/add_post.html'
 
-    def get_form_kwargs(self):
-        ip = self.request.META.get('REMOTE_ADDR', '')
-        form_kwargs = super(AddPostView, self).get_form_kwargs()
-        form_kwargs.update(dict(topic=self.topic, forum=self.forum, user=self.user,
-                       ip=ip, initial={}))
-        if 'quote_id' in self.request.GET:
-            try:
-                quote_id = int(self.request.GET.get('quote_id'))
-            except TypeError:
-                raise Http404
-            else:
-                post = get_object_or_404(Post, pk=quote_id)
-                quote = defaults.PYBB_QUOTE_ENGINES[defaults.PYBB_MARKUP](post.body, getattr(post.user, username_field))
-                form_kwargs['initial']['body'] = quote
-        if self.user.is_staff:
-            form_kwargs['initial']['login'] = getattr(self.user, username_field)
-        return form_kwargs
-
-    def get_context_data(self, **kwargs):
-        ctx = super(AddPostView, self).get_context_data(**kwargs)
-        ctx['forum'] = self.forum
-        ctx['topic'] = self.topic
-        return ctx
-
-    def get_success_url(self):
-        if (not self.request.user.is_authenticated()) and defaults.PYBB_PREMODERATION:
-            return reverse('pybb:index')
-        return super(AddPostView, self).get_success_url()
-
     @method_decorator(csrf_protect)
     def dispatch(self, request, *args, **kwargs):
         if request.user.is_authenticated():
@@ -359,8 +334,43 @@ class AddPostView(PostEditMixin, generic.CreateView):
             self.topic = get_object_or_404(perms.filter_topics(request.user, Topic.objects.all()), pk=kwargs['topic_id'])
             if not perms.may_create_post(self.user, self.topic):
                 raise PermissionDenied
+
+            self.quote = ''
+            if 'quote_id' in request.GET:
+                try:
+                    quote_id = int(request.GET.get('quote_id'))
+                except TypeError:
+                    raise Http404
+                else:
+                    post = get_object_or_404(Post, pk=quote_id)
+                    self.quote = defaults.PYBB_QUOTE_ENGINES[defaults.PYBB_MARKUP](post.body, getattr(post.user, username_field))
+
+                if self.quote and request.is_ajax():
+                    return HttpResponse(self.quote)
         return super(AddPostView, self).dispatch(request, *args, **kwargs)
 
+    def get_form_kwargs(self):
+        ip = self.request.META.get('REMOTE_ADDR', '')
+        form_kwargs = super(AddPostView, self).get_form_kwargs()
+        form_kwargs.update(dict(topic=self.topic, forum=self.forum, user=self.user,
+                           ip=ip, initial={}))
+        if getattr(self, 'quote', None):
+            form_kwargs['initial']['body'] = self.quote
+        if self.user.is_staff:
+            form_kwargs['initial']['login'] = getattr(self.user, username_field)
+        return form_kwargs
+
+    def get_context_data(self, **kwargs):
+        ctx = super(AddPostView, self).get_context_data(**kwargs)
+        ctx['forum'] = self.forum
+        ctx['topic'] = self.topic
+        return ctx
+
+    def get_success_url(self):
+        if (not self.request.user.is_authenticated()) and defaults.PYBB_PREMODERATION:
+            return reverse('pybb:index')
+        return super(AddPostView, self).get_success_url()
+
 
 class EditPostView(PostEditMixin, generic.UpdateView):
 
