@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+from test.example_thirdparty.forms import SignupFormWithCaptcha
+
 try:
     from django.conf.urls import patterns, include, url
 except ImportError:
@@ -11,12 +13,13 @@ from account.views import ChangePasswordView, SignupView, LoginView
 
 urlpatterns = patterns('',
     url(r'^admin/', include(admin.site.urls)),
-    url(r'^accounts/', include('account.urls')),
 
     # aliases to match original django-registration urls
     url(r"^accounts/password/$", ChangePasswordView.as_view(), name="auth_password_change"),
-    url(r"^accounts/signup/$", SignupView.as_view(), name="registration_register"),
+    url(r"^accounts/signup/$", SignupView.as_view(form_class=SignupFormWithCaptcha), name="registration_register"),
     url(r"^accounts/login/$", LoginView.as_view(), name="auth_login"),
 
+    url(r'^accounts/', include('account.urls')),
+    url(r'^captcha/', include('captcha.urls')),
     url(r'^', include('pybb.urls', namespace='pybb')),
 )
